@@ -108,7 +108,6 @@ import os, itertools
 from pprint import pprint
 from rdflib import Namespace
 from rdflib import plugin,RDF,RDFS,URIRef,BNode,Literal,Variable
-from rdflib import OWL
 from rdflib.Literal import _XSD_NS
 from rdflib.Identifier import Identifier
 from rdflib.util import first
@@ -576,19 +575,19 @@ class ClassNamespaceFactory(Namespace):
             return self.term(name)
 
 CLASS_RELATIONS = set(
-        OWL.resourceProperties
-    ).difference([OWL.onProperty,
-                  OWL.allValuesFrom,
-                  OWL.hasValue,
-                  OWL.someValuesFrom,
-                  OWL.inverseOf,
-                  OWL.imports,
-                  OWL.versionInfo,
-                  OWL.backwardCompatibleWith,
-                  OWL.incompatibleWith,
-                  OWL.unionOf,
-                  OWL.intersectionOf,
-                  OWL.oneOf])
+        OWL_NS.resourceProperties
+    ).difference([OWL_NS.onProperty,
+                  OWL_NS.allValuesFrom,
+                  OWL_NS.hasValue,
+                  OWL_NS.someValuesFrom,
+                  OWL_NS.inverseOf,
+                  OWL_NS.imports,
+                  OWL_NS.versionInfo,
+                  OWL_NS.backwardCompatibleWith,
+                  OWL_NS.incompatibleWith,
+                  OWL_NS.unionOf,
+                  OWL_NS.intersectionOf,
+                  OWL_NS.oneOf])
 
 def ComponentTerms(cls):
     """
@@ -1830,6 +1829,12 @@ class Property(AnnotatibleTerms):
         pass            
                                 
     range = property(_get_range, _set_range, _del_range)
+
+    def replace(self,other):
+        extension = []
+        for s,p,o in self.extent:
+            self.graph.add((s,propertyOrIdentifier(other),o))
+        self.graph.remove((None,self.identifier,None))
         
 def CommonNSBindings(graph,additionalNS={}):
     """
@@ -1842,10 +1847,10 @@ def CommonNSBindings(graph,additionalNS={}):
     for prefix,uri in additionalNS.items():
         namespace_manager.bind(prefix, uri, override=False)
     graph.namespace_manager = namespace_manager
-        
-def test():
-    import doctest
-    doctest.testmod()
 
 if __name__ == '__main__':
+
+    def test():
+        import doctest
+        doctest.testmod()
     test()
